import click
from fmdpy.api import query, getSongURLs
from fmdpy.download import *
from fmdpy import VERSION
import sys

@click.command()
@click.option('-c', "--count", default=30, help="Max Number of results")
@click.option('-f', "--fmt", default='opus', help="Format of the audio file. (opus is default)")
@click.option('-b', "--bitrate", default=250, help="Bitrate in kb, (250k is default)")
@click.option('-d', "--directory", default='./', help="Specify the folder.", type=click.Path(exists=True))
@click.option('-l', "--lyrics", help="Add lyrics", is_flag=True)
@click.option('-V', "--Version", help="display version", is_flag=True)
@click.argument('search', nargs=-1)
def fmdpy(count, search, fmt, bitrate, version, lyrics, directory):
    """Download music with metadata\n
    For multiple download you can use something like:\n
    "Download: 1, 2, 3, 5:8", (This will download 1, 2, 3, 5, 6, 7, 8)

    -f native: save to native container (ffmpeg not req., -b is ignored)
    """
    if version:
        print("fmdpy:", VERSION)
        sys.exit(0)
    search=' '.join(search)
    song_list = query(search, count)
    for i, s in enumerate(song_list):
        print(f'{i+1}) {s.title} [{s.artist}] ({s.year})')

    if len(song_list) > 0:
        download_pool = [];
        to_download = input("\nDownload: ")
        for indx in to_download.replace(' ', '').split(','):
            if ':' in indx:
                [l, u] = indx.split(':')
                [ download_pool.append(i - 1) for i in range(int(l), int(u)+1) ]
            elif '-' in indx:
                [l, u] = indx.split('-')
                [ download_pool.append(i - 1) for i in range(int(l), int(u)+1) ]
            else:
                download_pool.append(int(indx) - 1)

            for i in download_pool:
                s = song_list[i]
                print(f'{i+1}) {s.title} [{s.artist}] ({s.year})')
                getSongURLs(s)
                if not Dl(s, dlformat=fmt, bitrate=bitrate, addlyrics=lyrics, directory=directory):
                    print(f'Unable to download: {i+1}) {s.title} [{s.artist}] ({s.year})')
                print("\n")
    else:
        print(f"No result for: {search}")

if __name__ == '__main__':
    fmdpy()



import requests
import sys
import tempfile
import ffmpeg
import music_tag
import lyricsgenius
import os
from fmdpy import *

# download file
def dlf(url, file_name, dltext=""):
    with open(file_name, "wb") as f:
        response = requests.get(url, headers=headers, stream=True)
        total_length = response.headers.get('content-length')

        if total_length is None: # no content length header
            f.write(response.content)
        else:
            dl = 0
            total_length = int(total_length)
            for data in response.iter_content(chunk_size=4096):
                dl += len(data); f.write(data)
                done = int(50 * dl / total_length)
                sys.stdout.write("\r%s[%s%s](%.2f%%)" \
                        % (dltext, '=' * done, ' ' * (50-done), (dl/total_length)*100))
                sys.stdout.flush()
    print("\tdone.")

def getLyric(song_obj):
    genius = lyricsgenius.Genius(mnc(b'U1ZZR1lzTnlzQXNfVWwwVVVZcW1wMkhPT0EzdF9ZSjRILUJfTzA0cVRtekctVW94RUtlZEFVMGhlX3BwVmd5cg==').decode('utf-8'))
    song = genius.search_song(song_obj.title, song_obj.artist)
    if song:
        return song.lyrics


def Dl(song_obj, dlformat='opus', bitrate=250, addlyrics=0, directory="./"):
    if song_obj.url == "":
        return None
    tf_song = tempfile.NamedTemporaryFile(suffix='.mp4')
    dlf(song_obj.url, tf_song.name, "SONG:")

    tf_thumb = tempfile.NamedTemporaryFile(suffix='.jpg')
    dlf(song_obj.thumb_url, tf_thumb.name, "ART :")
    output_file=directory + f"/{song_obj.artist}-{song_obj.title}({song_obj.year})"\
            .replace(' ', '_').lower()

    if dlformat != 'native':
        output_file+=f".{dlformat}"
        sys.stdout.write("Convering to %s..." % dlformat)
        sys.stdout.flush()
        # convert to desired format.
        (
                ffmpeg
                .input(tf_song.name)
                .output(output_file, **{'b:a': f'{bitrate}k'})
                .global_args('-loglevel', 'error', '-vn')
                .run()
        )
        sys.stdout.write("done\n")
        sys.stdout.flush()
    else:
        output_file+='.mp4'
        if not os.path.isfile(output_file):
            with open(output_file, 'wb') as f:
                f.write(tf_song.read())
        else:
            print(f"[WARNING]: File {output_file + '.mp4'} exist, skipping")
            return False

    # add music tags
    sys.stdout.write("Adding Metadata...")
    sys.stdout.flush()
    f = music_tag.load_file(output_file)
    f['year'] = song_obj.year
    f.append_tag('title', song_obj.title)
    f.append_tag('artist', song_obj.artist)
    f.append_tag('album', song_obj.album)
    f.append_tag('comment', song_obj.copyright + ', downloaded using (https://github.com/Liupold/fmdpy)')
    f.append_tag('album', song_obj.album)
    f['artwork'] = tf_thumb.read()
    if addlyrics:
        song_lyric = getLyric(song_obj)
        if song_lyric:
            f['lyrics'] = song_lyric
    f.save()
    sys.stdout.write("done\n")
    sys.stdout.flush()
    return True




from dataclasses import dataclass

@dataclass
class Song:
    songid: str
    title: str = "Unknown"
    album: str = "Unknown"
    artist: str = "Unknown"
    year: str = "Unknown"
    copyright: str = "Unknown"
    url: str = ""
    thumb_url: str = ""
    raw_json = {}

    def geturl(self):
        pass




import setuptools
from fmdpy import VERSION


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="fmdpy",
    version=VERSION,
    author="Rohn Chatterjee",
    author_email="rohn.ch@gmail.com",
    description="Music Downloader",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Liupold/fmdpy",
    license='GPLv3',
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=['ffmpeg-python', 'click', 'music-tag>=0.4.3', 'requests', 'pillow', 'lyricsgenius', 'dataclasses'],
    entry_points = {
        'console_scripts': ['fmdpy=fmdpy.__main__:fmdpy'],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable"
    ],
    python_requires='>=3.6',
)



import requests
from fmdpy import headers, art
from fmdpy.song import Song


def parse_query(query_json):
    song_list = []
    for s in query_json['results']:
        song_id = s['id']
        song_title = s['title']
        song_year = s['year']
        song_album = s['more_info']['album']
        song_copyright = s['more_info']['copyright_text']
        if len(s['more_info']['artistMap']['primary_artists']) != 0:
            song_artist = s['more_info']['artistMap']['primary_artists'][0]['name']
        else:
            song_artist = "Unknown"
        song_ = Song(songid=song_id, \
                title=song_title, artist=song_artist, year=song_year, album=song_album, copyright=song_copyright)
        song_list.append(song_)
    return song_list

def query(query_text, max_results=5):
    if ("fmd" in query_text) or ("liupold" in query_text):
        print(art)

    r = requests.get(headers=headers, \
            url=f"https://www.jiosaavn.com/api.php?p=1&q={query_text.replace(' ', '+')}\
            &_format=json&_marker=0&api_version=4&ctx=wap6dot0\
            &n={max_results}&__call=search.getResults")
    return parse_query(r.json())

def getSongURLs(song_obj):
    r = requests.get(headers=headers, \
            url=f"https://www.jiosaavn.com/api.php?__call=song.getDetails&cc=in&_marker=0%3F_marker%3D0&_format=json&pids={song_obj.songid}")
    raw_json = r.json()[song_obj.songid]
    if 'media_preview_url' in raw_json.keys():
        song_obj.url = raw_json['media_preview_url'].\
                replace('https://preview.saavncdn.com/', 'https://aac.saavncdn.com/').\
                replace('_96_p.mp4', '_320.mp4')
        song_obj.thumb_url = raw_json['image'].replace('-150x150.jpg', '-500x500.jpg')

